@@ -263,6 +263,16 @@ If there are more arguments expected after the line and column numbers."
   (interactive)
   (lsp-clojure--refactoring-call "backward-barf"))
 
+(defun lsp-clojure-kill-sexpr ()
+  "Apply kill sexpr refactoring at point."
+  (interactive)
+  (lsp-clojure--refactoring-call "kill-sexp"))
+
+(defun lsp-clojure-raise-sexpr ()
+  "Apply raise refactoring at point."
+  (interactive)
+  (lsp-clojure--refactoring-call "raise-sexp"))
+
 (defun lsp-clojure-move-form (dest-filename)
   "Apply move-form refactoring at point to DEST-FILENAME."
   (interactive
